@@ -17,11 +17,14 @@ import inspect
 import logging as py_logging
 import os
 import time
+from typing import Any, Callable, Tuple, Type, Union   # noqa: H301
 
+from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import strutils
 
 from os_brick.i18n import _
+from os_brick.privileged import nvmeof as priv_nvme
 from os_brick.privileged import rootwrap as priv_rootwrap
 
 
@@ -30,7 +33,7 @@ CUSTOM_LINK_PREFIX = '/dev/disk/by-id/os-brick'
 _time_sleep = time.sleep
 
 
-def _sleep(duration):
+def _sleep(secs: float) -> None:
     """Helper class to make it easier to work around tenacity's sleep calls.
 
     Apparently we are all idiots for wanting to test our code here [0], so this
@@ -38,7 +41,7 @@ def _sleep(duration):
 
     [0] https://github.com/jd/tenacity/issues/25
     """
-    _time_sleep(duration)
+    _time_sleep(secs)
 
 
 time.sleep = _sleep
@@ -49,7 +52,27 @@ import tenacity  # noqa
 LOG = logging.getLogger(__name__)
 
 
-def retry(exceptions, interval=1, retries=3, backoff_rate=2):
+class retry_if_exit_code(tenacity.retry_if_exception):
+    """Retry on ProcessExecutionError specific exit codes."""
+    def __init__(self, codes: Union[int, Tuple[int, ...]]):
+        self.codes = (codes,) if isinstance(codes, int) else codes
+        super(retry_if_exit_code, self).__init__(self._check_exit_code)
+
+    def _check_exit_code(self, exc: Type[Exception]) -> bool:
+        return (bool(exc) and
+                isinstance(exc, processutils.ProcessExecutionError) and
+                exc.exit_code in self.codes)
+
+
+def retry(retry_param: Union[None,
+                             Type[Exception],
+                             Tuple[Type[Exception], ...],
+                             int,
+                             Tuple[int, ...]],
+          interval: float = 1,
+          retries: int = 3,
+          backoff_rate: float = 2,
+          retry: Callable = tenacity.retry_if_exception_type) -> Callable:
 
     if retries < 1:
         raise ValueError(_('Retries must be greater than or '
@@ -64,17 +87,17 @@ def retry(exceptions, interval=1, retries=3, backoff_rate=2):
                 after=tenacity.after_log(LOG, logging.DEBUG),
                 stop=tenacity.stop_after_attempt(retries),
                 reraise=True,
-                retry=tenacity.retry_if_exception_type(exceptions),
+                retry=retry(retry_param),
                 wait=tenacity.wait_exponential(
                     multiplier=interval, min=0, exp_base=backoff_rate))
-            return r.call(f, *args, **kwargs)
+            return r(f, *args, **kwargs)
 
         return _wrapper
 
     return _decorator
 
 
-def platform_matches(current_platform, connector_platform):
+def platform_matches(current_platform: str, connector_platform: str) -> bool:
     curr_p = current_platform.upper()
     conn_p = connector_platform.upper()
     if conn_p == 'ALL':
@@ -87,7 +110,7 @@ def platform_matches(current_platform, connector_platform):
     return False
 
 
-def os_matches(current_os, connector_os):
+def os_matches(current_os: str, connector_os: str) -> bool:
     curr_os = current_os.upper()
     conn_os = connector_os.upper()
     if conn_os == 'ALL':
@@ -101,7 +124,7 @@ def os_matches(current_os, connector_os):
     return False
 
 
-def merge_dict(dict1, dict2):
+def merge_dict(dict1: dict, dict2: dict) -> dict:
     """Try to safely merge 2 dictionaries."""
     if type(dict1) is not dict:
         raise Exception("dict1 is not a dictionary")
@@ -113,7 +136,7 @@ def merge_dict(dict1, dict2):
     return dict3
 
 
-def trace(f):
+def trace(f: Callable) -> Callable:
     """Trace calls to the decorated function.
 
     This decorator should always be defined as the outermost decorator so it
@@ -181,7 +204,7 @@ def trace(f):
     return trace_logging_wrapper
 
 
-def convert_str(text):
+def convert_str(text: Union[bytes, str]) -> str:
     """Convert to native string.
 
     Convert bytes and Unicode strings to native strings:
@@ -194,6 +217,17 @@ def convert_str(text):
         return text
 
 
+def get_host_nqn():
+    try:
+        with open('/etc/nvme/hostnqn', 'r') as f:
+            host_nqn = f.read().strip()
+    except IOError:
+        host_nqn = priv_nvme.create_hostnqn()
+    except Exception:
+        host_nqn = None
+    return host_nqn
+
+
 def _symlink_name_from_device_path(device_path):
     """Generate symlink absolute path for encrypted devices.
 
@@ -229,7 +263,8 @@ def _device_path_from_symlink(symlink):
     return symlink
 
 
-def connect_volume_prepare_result(func):
+def connect_volume_prepare_result(
+        func: Callable[[Any, dict], dict]) -> Callable[[Any, dict], dict]:
     """Decorator to prepare the result of connect_volume for encrypted volumes.
 
     WARNING: This decorator must be **before** any connect_volume locking
